@@ -17,67 +17,26 @@
 #   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 import os
+import xml.dom.minidom
 import matemenu
 import gi
-from gi.repository import Gtk, Gdk, GdkPixbuf
-from ConfigParser import ConfigParser
+from collections import Sequence
+from gi.repository import GLib, Gtk, Gdk, GdkPixbuf
 
-class DesktopParser(ConfigParser):
-	def __init__(self, filename=None, file_type='Application'):
-		ConfigParser.__init__(self)
-		self.filename = filename
-		self.file_type = file_type
-		if filename:
-			if len(self.read(filename)) == 0:
-				#file doesn't exist
-				self.add_section('Desktop Entry')
-		else:
-			self.add_section('Desktop Entry')
-		self._list_separator = ';'
+DESKTOP_GROUP = GLib.KEY_FILE_DESKTOP_GROUP
+KEY_FILE_FLAGS = GLib.KeyFileFlags.KEEP_COMMENTS | GLib.KeyFileFlags.KEEP_TRANSLATIONS
 
-	def optionxform(self, option):
-		#makes keys not be lowercase
-		return option
+def fillKeyFile(keyfile, items):
+	for key, item in items.iteritems():
+		if item is None:
+			continue
 
-	def get(self, option, locale=None):
-		locale_option = option + '[%s]' % locale
-		try:
-			value = ConfigParser.get(self, 'Desktop Entry', locale_option)
-		except:
-			try:
-				value = ConfigParser.get(self, 'Desktop Entry', option)
-			except:
-				return None
-		if self._list_separator in value:
-			value = value.split(self._list_separator)
-		if value == 'true':
-			value = True
-		if value == 'false':
-			value = False
-		return value
-
-	def set(self, option, value, locale=None):
-		if locale:
-			option = option + '[%s]' % locale
-		if value == True:
-			value = 'true'
-		if value == False:
-			value = 'false'
-		if isinstance(value, tuple) or isinstance(value, list):
-			value = self._list_separator.join(value) + ';'
-		ConfigParser.set(self, 'Desktop Entry', option, value)
-
-	def write(self, file_object):
-		file_object.write('[Desktop Entry]\n')
-		items = []
-		if not self.filename:
-			file_object.write('Encoding=UTF-8\n')
-			file_object.write('Type=' + str(self.file_type) + '\n')
-		for item in self.items('Desktop Entry'):
-			items.append(item)
-		items.sort()
-		for item in items:
-			file_object.write(item[0] + '=' + item[1] + '\n')
+		if isinstance(item, bool):
+			keyfile.set_boolean(DESKTOP_GROUP, key, item)
+		elif isinstance(item, Sequence):
+			keyfile.set_string_list(DESKTOP_GROUP, key, item)
+		elif isinstance(item, basestring):
+			keyfile.set_string(DESKTOP_GROUP, key, item)
 
 def getUniqueFileId(name, extension):
 	append = 0
@@ -124,75 +83,44 @@ def getUniqueUndoFile(filepath):
 			append += 1
 	return new_filepath
 
-def getUserMenuPath():
-	menu_dir = None
-	if os.environ.has_key('XDG_CONFIG_HOME'):
-		menu_dir = os.path.join(os.environ['XDG_CONFIG_HOME'], 'menus')
-	else:
-		menu_dir = os.path.join(os.environ['HOME'], '.config', 'menus')
-	#move .config out of the way if it's not a dir, it shouldn't be there
-	if os.path.isfile(os.path.split(menu_dir)[0]):
-		os.rename(os.path.split(menu_dir)[0], os.path.split(menu_dir)[0] + '.old')
-	if not os.path.isdir(menu_dir):
-		os.makedirs(menu_dir)
-	return menu_dir
-
 def getItemPath(file_id):
-	if os.environ.has_key('XDG_DATA_DIRS'):
-		for system_path in os.environ['XDG_DATA_DIRS'].split(':'):
-			file_path = os.path.join(system_path, 'applications', file_id)
-			if os.path.isfile(file_path):
-				return file_path
-	file_path = os.path.join('/', 'usr', 'share', 'applications', file_id)
+	for path in GLib.get_system_data_dirs():
+		file_path = os.path.join(path, 'applications', file_id)
 	if os.path.isfile(file_path):
 		return file_path
-	return False
+	return None
 
 def getUserItemPath():
-	item_dir = None
-	if os.environ.has_key('XDG_DATA_HOME'):
-		item_dir = os.path.join(os.environ['XDG_DATA_HOME'], 'applications')
-	else:
-		item_dir = os.path.join(os.environ['HOME'], '.local', 'share', 'applications')
+	item_dir = os.path.join(GLib.get_user_data_dir(), 'applications')
 	if not os.path.isdir(item_dir):
 		os.makedirs(item_dir)
 	return item_dir
 
 def getDirectoryPath(file_id):
-	home = getUserDirectoryPath()
-	file_path = os.path.join(home, file_id)
-	if os.path.isfile(file_path):
-		return file_path
-	if os.environ.has_key('XDG_DATA_DIRS'):
-		for system_path in os.environ['XDG_DATA_DIRS'].split(':'):
-			file_path = os.path.join(system_path, 'desktop-directories', file_id)
-			if os.path.isfile(file_path):
-				return file_path
-	file_path = os.path.join('/', 'usr', 'share', 'desktop-directories', file_id)
-	if os.path.isfile(file_path):
-		return file_path
-	return False
+	for path in GLib.get_system_data_dirs():
+		file_path = os.path.join(path, 'desktop-directories', file_id)
+		if os.path.isfile(file_path):
+			return file_path
+	return None
 
 def getUserDirectoryPath():
-	menu_dir = None
-	if os.environ.has_key('XDG_DATA_HOME'):
-		menu_dir = os.path.join(os.environ['XDG_DATA_HOME'], 'desktop-directories')
-	else:
-		menu_dir = os.path.join(os.environ['HOME'], '.local', 'share', 'desktop-directories')
+	menu_dir = os.path.join(GLib.get_user_data_dir(), 'desktop-directories')
 	if not os.path.isdir(menu_dir):
 		os.makedirs(menu_dir)
 	return menu_dir
 
-def getSystemMenuPath(file_name):
-	if os.environ.has_key('XDG_CONFIG_DIRS'):
-		for system_path in os.environ['XDG_CONFIG_DIRS'].split(':'):
-			file_path = os.path.join(system_path, 'menus', file_name)
-			if os.path.isfile(file_path):
-				return file_path
-	file_path = os.path.join('/', 'etc', 'xdg', 'menus', file_name)
-	if os.path.isfile(file_path):
-		return file_path
-	return False
+def getUserMenuPath():
+	menu_dir = os.path.join(GLib.get_user_config_dir(), 'menus')
+	if not os.path.isdir(menu_dir):
+		os.makedirs(menu_dir)
+	return menu_dir
+
+def getSystemMenuPath(file_id):
+	for path in GLib.get_system_config_dirs():
+		file_path = os.path.join(path, 'menus', file_id)
+		if os.path.isfile(file_path):
+			return file_path
+	return None
 
 def getUserMenuXml(tree):
 	system_file = getSystemMenuPath(tree.get_menu_file())
@@ -202,11 +130,9 @@ def getUserMenuXml(tree):
 	menu_xml += "<MergeFile type=\"parent\">" + system_file +	"</MergeFile>\n</Menu>\n"
 	return menu_xml
 
-def getIcon(item, for_properties=False):
+def getIcon(item):
 	pixbuf, path = None, None
-	if item == None:
-		if for_properties:
-			return None, None
+	if item is None:
 		return None
 	if isinstance(item, str):
 		iconName = item
@@ -225,9 +151,7 @@ def getIcon(item, for_properties=False):
 				path = iconName
 			except:
 				pass
-		if pixbuf == None:
-			if for_properties:
-				return None, None
+		if pixbuf is None:
 			if item.get_type() == matemenu.TYPE_DIRECTORY:
 				iconName = 'mate-fs-directory'
 			elif item.get_type() == matemenu.TYPE_ENTRY:
@@ -237,10 +161,20 @@ def getIcon(item, for_properties=False):
 				path = icon_theme.lookup_icon(iconName, 24, 0).get_filename()
 			except:
 				return None
-	if pixbuf == None:
+	if pixbuf is None:
 		return None
 	if pixbuf.get_width() != 24 or pixbuf.get_height() != 24:
 		pixbuf = pixbuf.scale_simple(24, 24, GdkPixbuf.InterpType.HYPER)
-	if for_properties:
-		return pixbuf, path
 	return pixbuf
+
+def removeWhitespaceNodes(node):
+	remove_list = []
+	for child in node.childNodes:
+		if child.nodeType == xml.dom.minidom.Node.TEXT_NODE:
+			child.data = child.data.strip()
+			if not child.data.strip():
+				remove_list.append(child)
+		elif child.hasChildNodes():
+			removeWhitespaceNodes(child)
+	for node in remove_list:
+		node.parentNode.removeChild(node)
